/*
 * Strings manipulations for commands
 */

#include "config.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <strings.h>
#include <ctype.h>
#include <time.h>
#include "defines.h"
#include "xmem.h"
#include "tools.h"


char *test_file(const char *dest_dir, const char *filename, int pos) {
    char *dest_file = NULL;
    char num[6];
    int len;

    bzero(num, 6);
    snprintf(num, 5, ".%d", pos);
    FREE(dest_file);
    len = strlen(dest_dir) + strlen(filename) + strlen(num) + 4;
    dest_file = xmalloc(len);
    sprintf(dest_file, "%s/%s%s", dest_dir, filename, pos > 0 ? num : "");
    printv(VERBOSE, "    => Vrification de la prsence de %s.\n", dest_file);
    if (!fexist(dest_file)) return dest_file;
    free(dest_file);
    return NULL;
}


char *get_dest_file(const char *dest_dir, const char *filename) {
    char *dest_file = NULL;
    int ok = FALSE;
    int pos = 0;

    while (! ok) {
        dest_file = test_file(dest_dir, filename, pos);
        ok = dest_file ? TRUE : FALSE;
        pos++;
    }
    return dest_file;
}


char *get_dated_cmd(char *c, const char *dir, const char *file, time_t ctime, const char *dstdir) {
    char      *cmd, *dir1, *dir2, *dir3, *dst_fil;
    char      *src_fil = get_filename(dir, file);
    int        len  = strlen(src_fil) + 7;
    struct tm *ft;

    ft   = localtime(&ctime);
    dir1 = test_dir(dstdir, ft->tm_year + 1900,  4);
    dir2 = test_dir(dir1,   ft->tm_mon + 1,      2);
    dir3 = test_dir(dir2,   ft->tm_mday,         2);
    dst_fil = get_dest_file(dir3, file);

    len += strlen(dst_fil);
    cmd  = xmalloc(len + 1);
    snprintf(cmd, len, "%s %s %s", c, src_fil, dst_fil);
    free(dst_fil);
    free(src_fil);
    free(dir1);
    free(dir2);
    free(dir3);

    return cmd;
}


char *get_normal_cmd(char *c, const char *dir, const char *file, const char *dstdir) {
    char *cmd;
    char *src_fil = get_filename(dir, file);
    char *dst_dir = test_dir(dstdir, 0, 0);
    char *dst_fil = get_dest_file(dst_dir, file);
    int   len = strlen(src_fil) + strlen(dst_fil) + 7;

    cmd  = xmalloc(len + 1);
    snprintf(cmd, len, "%s %s %s", c, src_fil, dst_fil);
    free(dst_fil);
    free(src_fil);
    free(dst_dir);

    return cmd;
}



