/*
 * Misc tools
 */


#include "config.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <string.h>
#include <strings.h>
#include <pwd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <errno.h>
#include <dirent.h>
#include <ctype.h>
#include <time.h>
#include <fcntl.h>
#ifdef OS_SunOS
#include <sys/varargs.h>
#else
#include <stdarg.h>
#endif
#include "defines.h"
#include "xmem.h"


extern char  **environ;
extern int     errno;
extern int     end_program;
extern int     verbose_lvl;
extern int     simulate;


char *robust_home() {
    if (getenv("HOME"))
        return getenv("HOME");
    else if (getenv("USER") && getpwnam(getenv("USER")))
        return getpwnam(getenv("USER"))->pw_dir;
    else if (getenv("LOGNAME") && getpwnam(getenv("LOGNAME")))
        return getpwnam(getenv("LOGNAME"))->pw_dir;
    else if (getpwuid(getuid()))
        return getpwuid(getuid())->pw_dir;
    return "/";
}


int my_system(char *command) {
    int pid, status;

    if (command == 0) return 1;
    pid = fork();
    if (pid == -1) return -1;
    if (pid == 0) {
        char *argv[4];
        argv[0] = "sh";
        argv[1] = "-c";
        argv[2] = command;
        argv[3] = 0;
        execve("/bin/sh", argv, environ);
        exit(127);
    }
    do {
        if (waitpid(pid, &status, 0) == -1) {
            if (errno != EINTR) return -1;
        } else
            return status;
    } while(1);
}


void signals_handler(int sig) {
    switch (sig) {
        case SIGINT:
        case SIGTERM:
        case SIGQUIT:
            end_program = TRUE;
            break;
    }
}


void printv(int level, const char *format, ...) {
    va_list  ap;
    FILE    *stream = stdout;

    va_start(ap, format);
    if (level == ERROR) stream = stderr;
    if (level <= verbose_lvl) {
        vfprintf(stream, format, ap);
        fflush(stream);
    }
    va_end(ap);
}


int fexist(const char *filename) {
    struct stat s;

    if (stat(filename, &s) == -1) return FALSE;
    return TRUE;
}


int watch_file(const char *filename, time_t *mtime, int force) {
    struct stat s;

    if (stat(filename, &s) == -1) {
        printv(ERROR, "Unable to stat %s !\n", filename);
        exit(2);
    }
    if (*mtime == 0) {
        *mtime = s.st_mtime;
        if (force) return TRUE;
        return FALSE;
    }
    if (s.st_mtime == *mtime)
        return FALSE;
    else
        *mtime = s.st_mtime;

    return TRUE;
}


char *get_filename(const char *dirname, const char *filename) {
    int   len  = strlen(dirname) + strlen(filename) + 2;
    char *name = xmalloc(len + 1);

    snprintf(name, len, "%s/%s", dirname, filename);
    return name;
}


time_t file_is_ok(const char *dirname, const char *filename) {
    struct stat  s;
    char        *name = get_filename(dirname, filename);
    time_t       ok = 0;

    if ((stat(name, &s) != -1) && (S_ISREG(s.st_mode))) ok = s.st_ctime;
    free(name);

    return ok;
}


char *test_dir(const char *dirbase, int date, int fsize) {
    int   len = strlen(dirbase) + fsize + 2;
    char *dirname = xmalloc(len + 1);
    struct stat s;

    if (fsize == 0) {
        snprintf(dirname, len, "%s", dirbase);
    } else {
        char format[19];
        snprintf(format, 18, "%%s/%%%02dd", fsize);
        snprintf(dirname, len, format, dirbase, date);
    }
    if (stat(dirname, &s) == -1) {
        printv(VERBOSE,
               "    -> Le rpertoire %s n'existe pas, cration...\n", dirname);
        if (! simulate) mkdir(dirname, 0755);
    } else {
        printv(VERBOSE,
               "    -> Le rpertoire %s existe dj.\n", dirname);
    }
    return dirname;
}


int get_nb_files(DIR *dir, const char *dirname) {
    struct dirent *dir_ent;
    int            nb = 0;

    while ((dir_ent = readdir(dir)) != NULL) {
        if (file_is_ok(dirname, dir_ent->d_name) != 0) nb++;
    }
    rewinddir(dir);

    return nb;
}



