/*
 * X11 datas display functions
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "config.h"
#include "defines.h"
#include "xmem.h"
#include "xwindow.h"
#include "xfunctions.h"

#define DEFAULTFONT "-adobe-helvetica-medium-r-normal-*-10-*-*-*-p-*-iso8859-1"
#define DEFAULTPOS 3

/* X11 datas */

char   *geom         = NULL;
char   *fontname     = NULL;
char   *fgcolor      = NULL;
char   *bgcolor      = NULL;
int     splash       = TRUE;

static X11App *app          = NULL;
static int     cx           = 0;
static int     cy           = 0;
static int     cw           = WIN_WIDTH;
static int     ch           = WIN_HEIGHT;
static unsigned long fg_color, bg_color;


void init_x11() {
    open_x11(NULL);
    fg_color = get_color(fgcolor ? fgcolor : "White");
    bg_color = get_color(bgcolor ? bgcolor : "Black");
    load_font(fontname ? fontname : DEFAULTFONT);
    cw = text_width("Dplacement de : 2004/04/06/dsc00000.jpg   100/100");
    ch = 4 * font_height();
    center_coords(cw, ch, &cx, &cy);
    app = create_app(geom, cx, cy, cw, ch, EVENTMASK, bg_color);
    set_properties(app, PACKAGE, PROGRAM);
    if (splash > 0) show_splashscreen();
}


void deinit_x11() {
    close_app(app);
    close_x11();
}


int display_window(int nbfiles) {
    display_app(app);
    sleep(1);
    return ask_for_cancel(nbfiles);
}


void close_window() {
    sleep(1);
    hide_app(app);
}


void show_splashscreen() {
    int y = DEFAULTPOS;

    display_app(app);
    sleep(1);
    set_color(fg_color);
    XDrawRectangle(display, app->window, gc,
            0, 0, app->wsize.width - 1, app->wsize.height - 1);
    print_center("CYBERPHOTOS", &y);
    print_center("", &y);
    print_center("CHARGEMENT EN COURS...", &y);
    update_x11();
    sleep(splash);
    hide_app(app);
}


void print_data(const char *data, int x, int *y) {
    xprint(app, data, x, *y);
    *y += 2 + font_height();
}


void print_center(const char *data, int *y) {
    int x = (app->wsize.width - text_width(data)) / 2;
    print_data(data, x, y);
}


void print_left(const char *data, int y) {
    int x = app->wsize.width - text_width(data) - 2;
    xprint(app, data, x, y);
}


void display_datas(const char *filename, const char *dest, int n, int m) {
    static int  x = 0;
    static char num[MAXSTRLEN + 1];
    static char src[MAXSTRLEN + 1];
    static char dst[MAXSTRLEN + 1];
    int y = DEFAULTPOS;

    x = text_width("Dplacement de ") + DEFAULTPOS;
    if (x != 0) {
        set_color(bg_color);
        print_data(src, x + 10, &y);
        print_data(dst, x + 10, &y);
        print_left(num, DEFAULTPOS);
        y = DEFAULTPOS;
    }
    bzero(num, MAXSTRLEN + 1); snprintf(num, MAXSTRLEN, "%d/%d", n, m);
    bzero(src, MAXSTRLEN + 1); snprintf(src, MAXSTRLEN, "%s", filename);
    bzero(dst, MAXSTRLEN + 1); snprintf(dst, MAXSTRLEN, "%s", dest);
    set_color(fg_color);
    XDrawRectangle(display, app->window, gc,
            0, 0, app->wsize.width - 1, app->wsize.height - 1);
    print_data("Dplacement de ", DEFAULTPOS, &y);
    xprint(app, ":", x, DEFAULTPOS);
    xprint(app, ":", x, y);
    print_data("vers", DEFAULTPOS, &y);
    y = DEFAULTPOS;
    print_left(num, y);
    print_data(src, x + 10, &y);
    print_data(dst, x + 10, &y);
    XFillRectangle(display, app->window, gc, DEFAULTPOS, y + 2,
            (app->wsize.width - 2*DEFAULTPOS) * n / m, font_height() - 1);
    update_x11();
}


void display_timeout(int max, int remain) {
    static int ny = 0;
    static char num[13];
    int y = DEFAULTPOS;
    char nb_photos[MAXSTRLEN + 1];
    char cancel[MAXSTRLEN + 1];

    if (ny != 0) {
        set_color(bg_color);
        print_center(num, &ny);
    }
    // Set texts
    bzero(num, 13);
    snprintf(num, 12, "%d secondes", remain);
    bzero(nb_photos, MAXSTRLEN + 1);
    snprintf(nb_photos, MAXSTRLEN, "Transfert automatique de %d photo%s dans",
            max, max > 1 ? "s" : "");
    bzero(cancel, MAXSTRLEN + 1);
    snprintf(cancel, MAXSTRLEN, "[ Taper '%c' pour annuler ]", STOPKEY);
    // display
    set_color(fg_color);
    XDrawRectangle(display, app->window, gc,
            0, 0, app->wsize.width - 1, app->wsize.height - 1);
    print_center(nb_photos, &y);
    ny = y;
    print_center(num, &y);
    print_center(cancel, &y);
    update_x11();
}


int ask_for_cancel(int max) {
    int end  = 10;
    int cont = TRUE;

    display_timeout(max, end);
    while (end != 0) {
        XEvent ev;
        if (smooth_nextevent_or_timeout(&ev, 1000)) {
            if (ev.type == KeyPress) {
                KeySym key;
                int    i;
                char   cars[10];
                i = XLookupString((XKeyEvent *) &ev, cars, 10, &key, NULL);
                if ((i == 1) && (cars[0] == STOPKEY)) {
                    cont = FALSE;
                    end  = 0;
                } else {
                    end = 0;
                }
            }
        } else {
            end--;
            display_timeout(max, end);
        }
    }
    clear_window(app);

    return cont;
}

