/*
 * rootapps main graphics functions
 */

#ifndef XWINDOW_H
#define XWINDOW_H

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>

typedef struct {
    Window     window;
    Pixmap     rootpixmap; /* saved pixmap */
    Pixmap     bgpixmap;   /* pixmap to write on */
    XSizeHints wsize;
} X11App;


extern Display *display;
extern GC       gc;


/* colors */
unsigned long get_color(const char *color_name);
void set_color(unsigned long color);
void set_color_name(const char *color_name);


/* fonts */
void load_font(const char *font_name);
Font get_font(const char *font_name);
void set_font(Font font);
int font_height();
int font_ascent();
int font_descent();
int text_width(const char *text);


/* Pixmaps and drawings */
Pixmap copy_pixmap(X11App *app, Pixmap source);
Pixmap root_pixmap(const char *desc);
Pixmap get_root_pixmap(X11App *app);
void pixmap_to_app(X11App *app, Pixmap source,
        int srcx, int srcy, int width, int height, int destx, int desty);
void draw_line(X11App *app, int x1, int y1, int x2, int y2);

/* Clean */
void clear_window(X11App *app);
void clear_bg(X11App *app);
void display_app(X11App *app);
void hide_app(X11App *app);

/* Writing */
void xprint(X11App *app, const char *data, int x, int y);
void update_x11();
void center_coords(int width, int height, int *x, int *y);

/* Create_app : mask == event mask, text == default text for size computing */
void open_x11(const char *displayname);
X11App *create_app(const char *geometry, int x, int y, int width, int height, int mask, unsigned long bg);
void set_properties(X11App *app, char *title, char *prog);
void close_app(X11App *app);
void close_x11();

/* Events */
int nextevent_or_timeout(XEvent *event, unsigned long miliseconds);
int smooth_nextevent_or_timeout(XEvent *event, unsigned long miliseconds);

#endif

